package de.uni_passau.fim.PADAS.group3.DataDash.Dataset;


import static org.mockito.BDDMockito.given;
import static org.mockito.Mockito.*;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.BeforeEach;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.data.domain.Pageable;

import de.uni_passau.fim.PADAS.group3.DataDash.category.Category;
import de.uni_passau.fim.PADAS.group3.DataDash.category.CategoryRepository;

import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

class DatasetServiceTest {

    @Mock
    private dataRepository datasetRepository;
    @Mock
    private CategoryRepository categoryRepository;

    @InjectMocks
    private DatasetService datasetService;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    void testGetDatasetById() { // Test for getDatasetById method
        UUID id = UUID.randomUUID();
        Dataset dataset = new Dataset();
        when(datasetRepository.getDatasetById(id)).thenReturn(dataset);
        assertEquals(dataset, datasetService.getDatasetById(id));
    }

    @Test
    void testAddDataset() { // Test for addDataset method
        Dataset dataset = new Dataset();
        when(datasetRepository.save(dataset)).thenReturn(dataset);
        assertEquals(dataset, datasetService.addDataset(dataset));
    }

    @Test
    void testVoteDataset() { // Test for voteDataset method
        UUID id = UUID.randomUUID();
        Dataset dataset = new Dataset();
        when(datasetRepository.getDatasetById(id)).thenReturn(dataset);
        datasetService.voteDataset(id, 5);
        verify(datasetRepository).save(dataset);
        assertEquals(1, dataset.getVotes());
        assertEquals(5, dataset.getRaiting());
    }

    @Test
    void testDeleteDataset() { // Test for deleteDataset method
        UUID id = UUID.randomUUID();
        Dataset dataset = new Dataset();
        when(datasetRepository.getDatasetById(id)).thenReturn(dataset);
        datasetService.deleteDataset(id);
        verify(datasetRepository).delete(dataset);
    }

    @Test
    void testUpvoteDataset() { // Test for upvoteDataset method
        UUID id = UUID.randomUUID();
        Dataset dataset = new Dataset();
        when(datasetRepository.getDatasetById(id)).thenReturn(dataset);
        datasetService.upvoteDataset(id);
        verify(datasetRepository).save(dataset);
        assertEquals(1, dataset.getUpvotes());
    }

    @Test
    void testDownvoteDataset() { // Test for downvoteDataset method
        UUID id = UUID.randomUUID();
        Dataset dataset = new Dataset();
        when(datasetRepository.getDatasetById(id)).thenReturn(dataset);
        datasetService.downvoteDataset(id);
        verify(datasetRepository).save(dataset);
        assertEquals(-1, dataset.getUpvotes());
    }

   
   
    @Test
    void testSearchByOptionalCriteria() { // Test for searchByOptionalCriteria method
        String search = "search";
        Type type = Type.DATASET;
        Pageable pageable = PageRequest.of(0, 10);
        List<Dataset> datasets = Arrays.asList(new Dataset("Title", "abst", "desc", "auth", null, null, Type.API, "MIT"), new Dataset("Title", "abst", "desc", "auth", null, null, Type.API, "MIT"));
        PageImpl<Dataset> s = new PageImpl<Dataset>(datasets);
        given(datasetRepository.searchByOptionalCriteria(Optional.ofNullable(search), Optional.ofNullable(type),pageable)).willReturn(s);
        assertEquals(s.getContent(), datasetService.searchByOptionalCriteria(search, "%",type.toString(), pageable).getContent());
    }

    @Test
    void testSearchByOptionalCriteriaWithCategory() { // Test for searchByOptionalCriteria method
        String search = "search";
        Type type = Type.DATASET;
        String category = UUID.randomUUID().toString();
        Category cat = new Category();
        Pageable pageable = PageRequest.of(0, 10);
        List<Dataset> datasets = Arrays.asList(new Dataset("Title", "abst", "desc", "auth", null, null, Type.API, "MIT"), new Dataset("Title", "abst", "desc", "auth", null, null, Type.API, "MIT"));
        PageImpl<Dataset> s = new PageImpl<Dataset>(datasets);
        given(categoryRepository.getCategoryById(UUID.fromString(category))).willReturn(cat);
        given(datasetRepository.searchByOptionalCriteriaWithCategory(Optional.ofNullable(search), cat ,Optional.ofNullable(type),pageable)).willReturn(s);
        assertEquals(s.getContent(), datasetService.searchByOptionalCriteria(search, category,type.toString(), pageable).getContent());
    }

    // Additional tests for other methods can be added following the same pattern.
}